import click

from stx.app import main


@click.command()
@click.argument('input_file')
@click.option('--watch', is_flag=True, default=False)
def cli(input_file: str, watch: bool):
    main(input_file, watch)


if __name__ == '__main__':
    cli()



from ._captured_text import CapturedText
from ._code_block import CodeBlock
from ._component import Component, DisplayMode
from ._composite import Composite
from ._content_box import ContentBox
from ._custom_text import CustomText
from ._figure import Figure
from ._function_call import FunctionCall
from ._image import Image
from ._layout import Layout
from ._link_text import LinkText
from ._list_block import ListBlock
from ._literal import Literal
from ._paragraph import Paragraph
from ._plain_text import PlainText
from ._section import Section
from ._separator import Separator
from ._styled_text import StyledText
from ._table import Table
from ._table_row import TableRow
from ._toc import TableOfContents, ElementReference



import os
import threading
import time

from watchdog.events import FileSystemEventHandler, FileSystemEvent
from watchdog.observers import Observer

from stx import logger
from stx.compiling.compiler import compile_document
from stx.document import Document
from stx.outputs import registry, OutputFile, OutputAction
from stx.utils.files import resolve_sibling
from stx.utils.debug import see


def read_version() -> str:
    version_file = resolve_sibling(__file__, '../version.txt')

    with open(version_file, 'r') as f:
        return f.read().strip()


version = read_version()  # TODO parse version
name = 'STX'
title = f'{name} {version}'


def process_file(input_file: str) -> Document:
    logger.info(f'Processing file {see(input_file)}...')

    document = compile_document(input_file)

    if len(document.actions) == 0:
        logger.warning('No actions were registered.')
    else:
        for action in document.actions:
            action.run()

    return document


def watch_file(input_file: str):
    ignored_files = set()

    def refresh():
        document = process_file(input_file)

        ignored_files.clear()

        for action in document.actions:
            if (isinstance(action, OutputAction)
                    and isinstance(action.target, OutputFile)):
                ignored_files.add(
                    os.path.abspath(action.target.file_path)
                )

    class Handler(FileSystemEventHandler):

        def __init__(self):
            self.lock = threading.Lock()

        def on_any_event(self, event: FileSystemEvent):
            if event.is_directory:
                return

            with self.lock:
                src_file = os.path.abspath(event.src_path)

                if src_file not in ignored_files:
                    logger.info(f'Detected change on: {src_file}')

                    refresh()

    refresh()

    root_dir = os.path.dirname(input_file)

    logger.info(f'Watching directory {see(root_dir)} for changes...')

    observer = Observer()
    observer.schedule(Handler(), root_dir, recursive=True)
    observer.start()
    try:
        while True:
            time.sleep(1)
    except KeyboardInterrupt:
        logger.info('Stopping watch...')
        observer.stop()
    observer.join()


def main(input_file: str, watch_mode=False):
    input_file = os.path.abspath(input_file)

    if watch_mode:
        watch_file(input_file)
    else:
        process_file(input_file)



from ._admonition import resolve_admonition
from ._admonition import resolve_warning
from ._admonition import resolve_information
from ._code import resolve_code
from ._custom_style import resolve_custom_style
from ._embed import resolve_embed
from ._layout import layout_function
from ._image import resolve_image
from ._line_feed import resolve_line_feed
from ._toc import resolve_toc



from typing import Optional

from stx.outputs.html5.escaping import html_escape
from stx.outputs.writer import Writer


class HtmlWriter(Writer):

    def _write_attributes(self, attributes: Optional[dict]):
        if attributes is not None and len(attributes) > 0:
            for attr_name, attr_value in attributes.items():
                self.write(' ')
                self.write(html_escape(attr_name))
                self.write('=')
                self.write('"')
                self.write(html_escape(str(attr_value)))
                self.write('"')

    def tag(
            self,
            name: str,
            attributes: Optional[dict] = None,
            inline=False):
        self.write('<')
        self.write(html_escape(name))

        self._write_attributes(attributes)

        self.write('>')

        if not inline:
            self.write('\n')

    def open_tag(
            self,
            name: str,
            attributes: Optional[dict] = None,
            inline=False):
        self.write('<')
        self.write(html_escape(name))

        self._write_attributes(attributes)

        self.write('>')

        if not inline:
            self.write('\n')
            self.indentation += 1

    def close_tag(self, name: str, inline=False):
        if not inline:
            self.indentation -= 1

            self.break_line()

        self.write('</')
        self.write(name)
        self.write('>')

        if not inline:
            self.write('\n')

    def text(self, content: str, disable_indentation=False):
        indentation0 = self.indentation

        if disable_indentation:
            self.indentation = 0

        self.write(html_escape(content))

        if disable_indentation:
            self.indentation = indentation0

    def comment(self, content: str):
        self.break_line()
        self.write('<!--')
        self.text(content)
        self.write('-->\n')



from __future__ import annotations

from io import TextIOWrapper
from typing import Optional, List

from stx.action import Action
from stx.components import Component

import sys
from typing import Optional, TextIO

from stx import logger
from stx.compiling.reading.location import Location
from stx.data_notation.values import Value, Empty
from stx.utils.files import resolve_sibling
from stx.utils.stx_error import StxError
from stx.utils.debug import see


class Document:

    def __init__(self, source_file):
        self.source_file = source_file
        self.title: Optional[str] = None
        self.author: Optional[str] = None
        self.format: Optional[str] = None
        self.encoding: Optional[str] = None
        self.header: Optional[Component] = None
        self.content: Optional[Component] = None
        self.footer: Optional[Component] = None
        self.stylesheets: List[str] = []
        self.actions: List[Action] = []



from typing import TextIO

from stx.outputs.html5.serializer import document_to_html
from stx.outputs.output_action import OutputAction


class HtmlOutputAction(OutputAction):

    def dump(self, out: TextIO):
        html = document_to_html(self.document)

        for tag in html:
            tag.render(out)

        # TODO implement pretty-print



from __future__ import annotations

from typing import List, TextIO

from ._component import Component, DisplayMode
from ..compiling.reading.location import Location
from ..data_notation.values import Value
from ..utils.tracked_dict import TrackedDict


# TODO this component may be replaced by Composite

class Paragraph(Component):

    def __init__(self, location: Location, contents: List[Component]):
        self.location = location
        self.contents = contents

    @property
    def display_mode(self) -> DisplayMode:
        return DisplayMode.BLOCK

    def is_rich(self) -> bool:
        for c in self.contents:
            if c.is_rich():
                return True
        return False

    def write_text(self, output: TextIO):
        for component in self.contents:
            component.write_text(output)

    def get_children(self) -> List[Component]:
        return self.contents

    def apply_advanced_attributes(self, attributes: TrackedDict[str, Value]):
        pass



import json
from typing import TextIO

from stx.outputs.json.serializer import document_to_json
from stx.outputs.output_action import OutputAction


class JsonOutputAction(OutputAction):

    def dump(self, out: TextIO):
        d = document_to_json(self.document)

        # TODO implement options
        # if output.options.to_dict():
        indent = 2
        # else:
            # indent = None

        json.dump(d, out, indent=indent)
